package boshclient_test

import (
	"encoding/json"
	"net/url"

	yaml "gopkg.in/yaml.v2"

	. "github.com/onsi/ginkgo"
	. "github.com/onsi/gomega"

	"github.com/pivotal-cf/on-demand-service-broker/boshclient"
	"github.com/pivotal-cf/on-demand-service-broker/mockbosh"
	"github.com/pivotal-cf/on-demand-services-sdk/bosh"
)

var _ = Describe("HasChanges", func() {
	var (
		deploymentName = "deployment-name"
		manifest       = bosh.BoshManifest{Name: "foo"}
		manifestBytes  []byte
	)

	BeforeEach(func() {
		var err error
		manifestBytes, err = yaml.Marshal(manifest)
		Expect(err).NotTo(HaveOccurred())
	})

	Context("when the proposed manifest causes changes", func() {
		BeforeEach(func() {
			director.VerifyAndMock(
				mockbosh.Diff(deploymentName).WithManifest(manifest).RespondsWithDiff(mockbosh.DeploymentDiff{Diff: [][]interface{}{
					{
						`super-secret: "redacted"`,
						"removed",
					},
				}}),
			)
		})

		It("returns true", func() {
			hasChanges, err := c.HasChanges(deploymentName, manifestBytes, logger)

			Expect(err).NotTo(HaveOccurred())
			Expect(hasChanges).To(BeTrue())
		})
	})

	Context("when the proposed manifest does not cause changes", func() {
		BeforeEach(func() {
			director.VerifyAndMock(
				mockbosh.Diff(deploymentName).WithManifest(manifest).RespondsWithNoDiff(),
			)
		})

		It("returns false", func() {
			hasChanges, err := c.HasChanges(deploymentName, manifestBytes, logger)

			Expect(err).NotTo(HaveOccurred())
			Expect(hasChanges).To(BeFalse())
		})
	})

	Context("when the deployment name is invalid", func() {
		It("returns an error", func() {
			_, err := c.HasChanges("$#%$#%#$^%&$#^$%^U%^&*$", manifestBytes, logger)
			Expect(err).To(BeAssignableToTypeOf(new(url.Error)))
		})
	})

	Context("when Bosh responds with an invalid diff response", func() {
		BeforeEach(func() {
			director.VerifyAndMock(
				mockbosh.Diff(deploymentName).WithManifest(manifest).RespondsWith("invalid"),
			)
		})

		It("returns an error", func() {
			_, err := c.HasChanges(deploymentName, manifestBytes, logger)
			Expect(err).To(BeAssignableToTypeOf(new(json.SyntaxError)))
		})
	})

	Context("when the bosh responds with an unexpected status code", func() {
		BeforeEach(func() {
			director.VerifyAndMock(
				mockbosh.Diff(deploymentName).WithManifest(manifest).Fails("no diff for you"),
			)
		})

		It("returns an error", func() {
			_, err := c.HasChanges(deploymentName, manifestBytes, logger)
			Expect(err.Error()).To(ContainSubstring("no diff for you"))
		})
	})

	Context("when bosh request fails", func() {
		BeforeEach(func() {
			director.Close()
		})

		It("returns an error", func() {
			_, err := c.HasChanges(deploymentName, manifestBytes, logger)
			Expect(err).To(BeAssignableToTypeOf(boshclient.RequestError{}))
		})
	})
})
